#! /usr/bin/ruby

$".push 'mkmf.rb'

if ARGV[0] == 'static'
  $force_static = true
  ARGV.shift
elsif ARGV[0] == 'install'
  $install = true
  $destdir = ARGV[1] || ''
  ARGV.shift
elsif ARGV[0] == 'clean'
  $clean = true
  ARGV.shift
end

SRC_EXT = ["c", "cc", "m", "cxx", "cpp", "C"]
$extlist = []

$includedir = "${prefix}/include".gsub(/\$\{prefix\}|\$\(prefix\)/,'/usr')

$top_srcdir = ".."
if $top_srcdir !~ "^/"
  # get absolute path
  $top_srcdir = File.expand_path($top_srcdir)
end
# get absolute path
$topdir = File.expand_path("..")

$:.push $top_srcdir
$:.push $top_srcdir+"/lib"

require 'find'

def older(file1, file2)
  if !File.exist?(file1) then
    return true
  end
  if !File.exist?(file2) then
    return false
  end
  if File.mtime(file1) < File.mtime(file2)
    return true
  end
  return false
end

if RUBY_PLATFORM == "m68k-human"
  CFLAGS = "-O3 -fomit-frame-pointer -fno-exceptions -fno-rtti -pipe -s -mpentium -mcpu=pentium -march=pentium -ffast-math -fexpensive-optimizations".gsub(/-c..-stack=[0-9]+ */, '')
else
  CFLAGS = "-O3 -fomit-frame-pointer -fno-exceptions -fno-rtti -pipe -s -mpentium -mcpu=pentium -march=pentium -ffast-math -fexpensive-optimizations"
end
LINK = "gcc -o conftest -I#$topdir -I#$top_srcdir #{CFLAGS} -I#$includedir -rdynamic %s %s conftest.c %s %s -ldl -lcrypt -lm "
CPP = "gcc -E  -I#$topdir -I#$top_srcdir #{CFLAGS} -I#$includedir %s %s conftest.c"

if /cygwin|mswin32|djgpp|mingw32|m68k-human|i386-os2_emx/i =~ RUBY_PLATFORM
  $null = open("nul", "w")
else
  $null = open("/dev/null", "w")
end

$orgerr = $stderr.dup
$orgout = $stdout.dup
def xsystem command
  if $DEBUG
    puts command
    return system(command)
  end
  $stderr.reopen($null) 
  $stdout.reopen($null) 
  r = system(command)
  $stderr.reopen($orgerr)
  $stdout.reopen($orgout)
  return r
end

def try_link0(src, opt="")
  cfile = open("conftest.c", "w")
  cfile.print src
  cfile.close
  xsystem(format(LINK, $CFLAGS, $LDFLAGS, opt, $LOCAL_LIBS))
end

def try_link(src, opt="")
  begin
    try_link0(src, opt)
  ensure
    system "rm -f conftest*"
  end
end

def try_cpp(src, opt="")
  cfile = open("conftest.c", "w")
  cfile.print src
  cfile.close
  begin
    xsystem(format(CPP, $CFLAGS, opt))
  ensure
    system "rm -f conftest*"
  end
end

def egrep_cpp(pat, src, opt="")
  cfile = open("conftest.c", "w")
  cfile.print src
  cfile.close
  begin
    xsystem(format(CPP+"|egrep #{pat}", $CFLAGS, opt))
  ensure
    system "rm -f conftest*"
  end
end

def try_run(src, opt="")
  begin
    if try_link0(src, opt)
      if xsystem("./conftest")
	true
      else
	false
      end
    else
      nil
    end
  ensure
    system "rm -f conftest*"
  end
end

def install_rb(mfile, srcdir = nil)
  libdir = "lib"
  libdir = srcdir + "/" + libdir if srcdir
  path = []
  dir = []
  Find.find(libdir) do |f|
    next unless /\.rb$/ =~ f
    f = f[libdir.length+1..-1]
    path.push f
    dir |= File.dirname(f)
  end
  for f in dir
    next if f == "."
    mfile.printf "\t@$(RUBY) -r ftools -e 'File::makedirs(*ARGV)' $(DESTDIR)$(pkglibdir)/%s\n", f
  end
  for f in path
    mfile.printf "\t@$(RUBY) -r ftools -e 'File::install(ARGV[0], ARGV[1], 0644, true)' $(srcdir)/lib/%s $(DESTDIR)$(pkglibdir)/%s\n", f, f
  end
end

def append_library(libs, lib)
  if /mswin32/ =~ RUBY_PLATFORM
    lib + ".lib " + libs
  else
    "-l" + lib + " " + libs
  end
end

def have_library(lib, func="main")
  if func && func != ""
    libs = append_library($libs, lib)
    if /mswin32/ =~ RUBY_PLATFORM
      r = try_link(<<"SRC", libs)
#include <windows.h>
#include <winsock.h>
int main() { return 0; }
int t() { #{func}(); return 0; }
SRC
      unless r
        r = try_link(<<"SRC", libs)
#include <windows.h>
#include <winsock.h>
int main() { return 0; }
int t() { void ((*p)()); p = (void ((*)()))#{func}; return 0; }
SRC
      end
    else
      r = try_link(<<"SRC", libs)
int main() { return 0; }
int t() { #{func}(); return 0; }
SRC
    end
    unless r
      return false
    end
  else
    libs = append_library($libs, lib)
  end

  $libs = libs
  return true
end

def find_library(lib, func, *paths)
  ldflags = $LDFLAGS
  libs = append_library($libs, lib)
  until try_link(<<"SRC", libs)
int main() { return 0; }
int t() { #{func}(); return 0; }
SRC
    if paths.size == 0
      $LDFLAGS = ldflags
      return false
    end
    $LDFLAGS = ldflags + " -L"+paths.shift
  end
  $libs = libs
  return true
end

def have_func(func)
  libs = $libs

  if /mswin32/ =~ RUBY_PLATFORM
    r = try_link(<<"SRC", libs)
#include <windows.h>
#include <winsock.h>
int main() { return 0; }
int t() { #{func}(); return 0; }
SRC
    unless r
      r = try_link(<<"SRC", libs)
#include <windows.h>
#include <winsock.h>
int main() { return 0; }
int t() { void ((*p)()); p = (void ((*)()))#{func}; return 0; }
SRC
    end
  else
    r = try_link(<<"SRC", libs)
int main() { return 0; }
int t() { #{func}(); return 0; }
SRC
  end
  unless r
    return false
  end
  $defs.push(format("-DHAVE_%s", func.upcase))
  return true
end

def have_header(header)
  unless try_cpp(<<"SRC")
#include <#{header}>
SRC
    return false
  end
  header.tr!("a-z./\055", "A-Z___")
  $defs.push(format("-DHAVE_%s", header))
  return true
end

def arg_config(config, default=nil)
  unless defined? $configure_args
    $configure_args = {}
    args = " i586-mandrake-linux --prefix=/usr --with-default-kcode=none"
    if /mswin32/ =~ RUBY_PLATFORM and ENV["CONFIGURE_ARGS"]
      args = args + " " + ENV["CONFIGURE_ARGS"]
    end
    for arg in args.split
      next unless /^--/ =~ arg
      if /=/ =~ arg
	$configure_args[$`] = $'
      else
	$configure_args[arg] = true
      end
    end
  end
  $configure_args.fetch(config, default)
end

def with_config(config, default=nil)
  unless /^--with-/ =~ config
    config = '--with-' + config
  end
  arg_config(config, default)
end

def enable_config(config, default=nil)
  if arg_config("--enable-"+config, default)
    true
  elsif arg_config("--disable-"+config, false)
    false
  else
    default
  end
end

def create_header()
  if $defs.length > 0
    hfile = open("extconf.h", "w")
    for line in $defs
      line =~ /^-D(.*)/
      hfile.printf "#define %s 1\n", $1
    end
    hfile.close
  end
end

def dir_config(target)
  dir = with_config("%s-dir"%target)
  if dir
    idir = " -I"+dir+"/include"
    ldir = " -L"+dir+"/lib"
  end
  unless idir
    dir = with_config("%s-include"%target)
    idir = " -I"+dir if dir
  end
  unless ldir
    dir = with_config("%s-lib"%target)
    ldir = " -L"+dir if dir
  end

  $CFLAGS += idir if idir
  $LDFLAGS += ldir if ldir
end

def create_makefile(target)
  $target = target

  system "rm -f conftest*"
  if "so" == $OBJEXT
    libs = $libs.split
    for lib in libs
      lib.sub!(/-l(.*)/, %%"lib\\1.#{$LIBEXT}"%)
    end
    $defs.push(format("-DEXTLIB='%s'", libs.join(",")))
  end

  $DLDFLAGS = ''

  if RUBY_PLATFORM =~ /beos/
    $libs = $libs + " -lruby"
    $DLDFLAGS = $DLDFLAGS + " -L" + $topdir
  end

  defflag = ''
  if RUBY_PLATFORM =~ /cygwin/ and not $static
    if File.exist? target + ".def"
      defflag = "--def=" + target + ".def"
    end
    $libs = $libs + " $(LIBRUBY_A)"
    $DLDFLAGS = $DLDFLAGS + " -L" + $topdir
  end

  $srcdir = $top_srcdir + "/ext/" + $mdir
  mfile = open("Makefile", "w")
  mfile.printf "\
SHELL = /bin/sh

#### Start of system configuration section. ####

srcdir = #{$srcdir}

topdir = #{$topdir}
hdrdir = #{$top_srcdir}
DESTDIR = 

CC = gcc

CFLAGS   = %s -I$(topdir) -I$(hdrdir) #{CFLAGS} #$CFLAGS -I${prefix}/include %s
DLDFLAGS = #$DLDFLAGS #$LDFLAGS
LDSHARED = gcc -shared #{defflag}
", if $static then "" else "-fPIC" end, $defs.join(" ")

  mfile.printf "\

RUBY_INSTALL_NAME = ruby

prefix = /usr
exec_prefix = ${prefix}
libdir = ${exec_prefix}/lib
#pkglibdir = $(libdir)/$(RUBY_INSTALL_NAME)/1.5
pkglibdir = $(libdir)/ruby/1.5
archdir = $(pkglibdir)/i586-linux-gnu


#### End of system configuration section. ####

"
  mfile.printf "LOCAL_LIBS = %s %s\n", $LOCAL_LIBS, $local_flags
  mfile.printf "LIBS = %s\n", $libs
  mfile.printf "OBJS = "
  if !$objs then
    $objs = []
    for f in Dir["#{$top_srcdir}/ext/#{$mdir}/*.{#{SRC_EXT.join(%q{,})}}"]
      f = File.basename(f)
      f.sub!(/(#{SRC_EXT.join(%q{|})})$/, $OBJEXT)
      $objs.push f
    end
  end
  mfile.printf $objs.join(" ")
  mfile.printf "\n"

  ruby_interpreter = "$(topdir)/miniruby"
  if /mswin32/ =~ RUBY_PLATFORM
    ruby_interpreter = $topdir + "/miniruby"
    ruby_interpreter.gsub!("/", "\\")
  end

  mfile.printf <<EOS
TARGET = #{target}
DLLIB = $(TARGET).#{$static ? $LIBEXT : "so"}

RUBY = #{ruby_interpreter} -I$(topdir) -I$(hdrdir)/lib

EXEEXT = 

all:		$(DLLIB)

clean:;		@rm -f *.#{$OBJEXT} *.so *.sl *.#{$LIBEXT} $(DLLIB)
		@rm -f *.ilk *.exp *.pdb *.bak
		@rm -f Makefile extconf.h conftest.*
		@rm -f core ruby$(EXEEXT) *~

realclean:	clean
EOS

  mfile.printf <<EOS

install:
	@$(RUBY) -r ftools -e 'File::makedirs(*ARGV)' $(DESTDIR)$(libdir) $(DESTDIR)$(pkglibdir) $(DESTDIR)$(archdir)
EOS
  unless $static
    mfile.printf "\
	@$(RUBY) -r ftools -e 'File::install(ARGV[0], ARGV[1], 0555, true)' $(DLLIB) $(DESTDIR)$(archdir)/$(DLLIB)
"
  end
  install_rb(mfile, $srcdir)
  mfile.printf "\n"

  if $static
    if "ar" =~ /^lib\b/i
      mfile.printf "\
$(DLLIB): $(OBJS)
	ar /OUT:$(DLLIB) $(OBJS)
"
    else
      mfile.printf "\
$(DLLIB): $(OBJS)
	ar cru $(DLLIB) $(OBJS)
	@-ranlib $(DLLIB) 2> /dev/null || true
"
    end
  elsif "so" != $OBJEXT
    mfile.printf "\
$(DLLIB): $(OBJS)
	$(LDSHARED) $(DLDFLAGS) -o $(DLLIB) $(OBJS) $(LIBS) $(LOCAL_LIBS)
"
  elsif RUBY_PLATFORM == "m68k-human"
    mfile.printf "\
$(DLLIB): $(OBJS)
	ar cru $(DLLIB) $(OBJS)
"
  else
    mfile.printf "\
$(DLLIB): $(OBJS)
	ld $(DLDFLAGS) -r -o $(DLLIB) $(OBJS)
"
  end

  if File.exist?("#{$srcdir}/depend")
    dfile = open("#{$srcdir}/depend", "r")
    mfile.printf "###\n"
    while line = dfile.gets()
      line.gsub!(/\.o/, ".#{$OBJEXT}")
      mfile.printf "%s", line.gsub('\$\(hdrdir\)/config.h', '$(topdir)/config.h')
    end
    dfile.close
  end
  mfile.close
  
  if RUBY_PLATFORM =~ /beos/
    if RUBY_PLATFORM =~ /^powerpc/ then
      deffilename = "ruby.exp"
    else
      deffilename = "ruby.def"
    end
    print "creating #{deffilename}\n"
    open(deffilename, "w") do |file|
      file.print("EXPORTS\n") if RUBY_PLATFORM =~ /^i/
      file.print("Init_#{target}\n")
    end
  end
end

def extmake(target)
  if $force_static or $static_ext[target]
    $static = target
  else
    $static = false
  end

  unless $install or $clean
    return if $nodynamic and not $static
  end

  $OBJEXT = "o"
  $LIBEXT = "a"
  $objs = nil
  $local_flags = ""
  case RUBY_PLATFORM
  when /cygwin|beos|openstep|nextstep|rhapsody|i386-os2_emx/
    $libs = ""
  when /mswin32/
    $LIBEXT = "lib"
    $libs = ""
    $local_flags = "$(topdir)/rubymw.lib -link /EXPORT:Init_$(TARGET)"
  else
    $libs = "-lc"
  end
  $LOCAL_LIBS = ""		# to be assigned in extconf.rb
  dir = with_config("opt-dir")
  if dir
    idir = "-I"+dir+"/include"
    ldir = "-L"+dir+"/lib"
  end
  unless idir
    dir = with_config("opt-include")
    idir = "-I"+dir if dir
  end
  unless ldir
    dir = with_config("opt-lib")
    ldir = "-L"+dir if dir
  end

  $CFLAGS = idir || ""
  $LDFLAGS = ldir || ""

  begin
    system "mkdir", target unless File.directory?(target)
    Dir.chdir target
    $target = target
    $mdir = target
    unless $install or $clean
      if $static_ext.size > 0 ||
	!File.exist?("./Makefile") ||
	older("./Makefile", "#{$top_srcdir}/ext/Setup") ||
	older("./Makefile", "../extmk.rb") ||
	older("./Makefile", "#{$top_srcdir}/ext/#{target}/makefile.rb") ||
	older("./Makefile", "#{$top_srcdir}/ext/#{target}/extconf.rb")
      then
	$defs = []
	if File.exist?("#{$top_srcdir}/ext/#{target}/makefile.rb")
	  load "#{$top_srcdir}/ext/#{target}/makefile.rb"
	elsif File.exist?("#{$top_srcdir}/ext/#{target}/extconf.rb")
	  load "#{$top_srcdir}/ext/#{target}/extconf.rb"
	else
	  create_makefile(target)
	end
      end
    end
    if File.exist?("./Makefile")
      if $static
 	$extlist.push [$static,$target]
      end
      if $install
	system "#{$make} install DESTDIR=#{$destdir}"
      elsif $clean
	system "#{$make} clean"
      else
	system "#{$make} all" or exit
      end
    end
    if $static
      $extlibs ||= ""
      $extlibs += " " + $LDFLAGS unless $LDFLAGS == ""
      $extlibs += " " + $libs unless $libs == ""
      $extlibs += " " + $LOCAL_LIBS unless $LOCAL_LIBS == ""
    end
  ensure
    system "rm -f conftest*"
    Dir.chdir ".."
  end
end

$make = ENV["MAKE"]
$make ||= with_config("make-prog", "make")

# get static-link modules
$static_ext = {}
for setup in ["Setup", "#{$top_srcdir}/ext/Setup"]
  if File.file? setup
    f = open(setup) 
    while f.gets()
      $_.chomp!
      sub!(/#.*$/, '')
      next if /^\s*$/
      if /^option +nodynamic/
	$nodynamic = true
	next
      end
      target = $_.split[0]
      target = target.downcase if /mswin32/ =~ RUBY_PLATFORM
      $static_ext[target] = true
    end
    f.close
    break
  end
end

for d in Dir["#{$top_srcdir}/ext/*"]
  File.directory?(d) || next
  File.file?(d + "/MANIFEST") || next
  
  d = File.basename(d)
  if $install
    print "installing ", d, "\n"
  elsif $clean
    print "cleaning ", d, "\n"
  else
    print "compiling ", d, "\n"
    if RUBY_PLATFORM =~ /-aix/ and older("../ruby.imp", "../miniruby")
      load "#{$top_srcdir}/ext/aix_mksym.rb"
    end
  end
  extmake(d)
end

if $install or $clean
  Dir.chdir ".."
  exit
end
$extinit = "" unless $extinit

ruby = "ruby"
miniruby = "miniruby"

$extobjs = "" unless $extobjs
if $extlist.size > 0
  for s,t in $extlist
    f = format("%s/%s.%s", s, t, $LIBEXT)
    if File.exist?(f)
      $extinit += format("\
\tInit_%s();\n\
\trb_provide(\"%s.so\");\n\
", t, t)
      $extobjs += "ext/"
      $extobjs += f
      $extobjs += " "
    else
      false
    end
  end

  if older("extinit.c", "#{$top_srcdir}/ext/Setup")
    f = open("extinit.c", "w")
    f.printf "void Init_ext() {\n"
    f.printf $extinit
    f.printf "}\n"
    f.close
  end
  if older("extinit.#{$OBJEXT}", "extinit.c")
    cmd = "gcc " + CFLAGS + " -c extinit.c"
    print cmd, "\n"
    system cmd or exit 1
  end

  Dir.chdir ".."

  if older(ruby, "#{$top_srcdir}/ext/Setup") or older(ruby, miniruby)
    system("rm -f #{ruby}")
  end

  $extobjs = "ext/extinit.#{$OBJEXT} " + $extobjs
  if RUBY_PLATFORM =~ /m68k-human|beos/
    $extlibs.gsub!("-L/usr/local/lib", "") if $extlibs
  end
  system format(%[#{$make} #{ruby} EXTOBJS="%s" EXTLIBS="%s"], $extobjs, $extlibs)
else
  Dir.chdir ".."
  if older(ruby, miniruby)
    system("rm -f #{ruby}")
    system("#{$make} #{ruby}")
  end
end

#Local variables:
# mode: ruby
#end:
